import React from "react";
import { INewsCategory } from "@interfaces/INews";
import { Pill } from "@components/Pill";
import { Link } from "@components/Link";
import { useMediaQuery } from "@hooks/useMediaQuery";

export interface HeroBoxProps {
  title: string;
  description: string;
  selfAligned?: "left" | "right" | "center";
  categories?: INewsCategory[];
  className?: string;
  transparentOnMobile?: boolean;
  target?: string;
}

export const HeroBox: React.FC<HeroBoxProps> = ({
  title,
  description,
  categories,
  className = "",
  selfAligned = "left",
  transparentOnMobile,
  target,
}) => {
  const mainStyle = transparentOnMobile ? "transparent" : "default";
  const isTabletOrDesktopSize = useMediaQuery("(min-width: 1024px)");
  const pillType = isTabletOrDesktopSize ? "dark" : "default";
  const POSITION_MAPPING: Record<string, string> = {
    left: "sm:left-0 sm:right-[unset] sm:translate-x-[unset]",
    right: "sm:right-0 sm:left-[unset] sm:translate-x-[unset]",
    center: "sm:left-1/2 sm:right-[unset] sm:-translate-x-1/2",
  };
  const STYLE_MAPPING: Record<string, Record<string, string>> = {
    default: {
      outerContainer:
        "group w-[75%] sm:w-2/3 lg:w-[45%] xl:w-2/5 2xl:w-[32%] absolute z-20 bottom-8 sm:bottom-[unset] left-1/2 transform -translate-x-1/2 sm:p-[inherit] sm:top-1/2 sm:-translate-y-2/4",
      innerContainer: "flex flex-col bg-pearl-800 p-8",
      headingContainer: "",
      heading:
        "font-karla text-2xl sm:text-3xl text-white multi-line-2-white sm:multi-line-3-white group-hover:multi-line-underline leading-8 sm:leading-8 font-extrabold tracking-tight",
      paragraph:
        "mt-3 sm:mt-2 font-plex text-lg text-white leading-6 sm:leading-7 font-normal",
      pillContainer: "mt-8 sm:mt-7 flex flex-wrap gap-2",
    },
    transparent: {
      outerContainer:
        "group w-full sm:w-2/3 lg:w-[60%] xl:w-2/5 2xl:w-[32%] absolute z-20 bottom-8 left-0 right-0 sm:p-[inherit] xl:top-1/2 xl:bottom-[unset] xl:-translate-y-1/2",
      innerContainer: "flex flex-col bg-transparent lg:bg-pearl-800 p-4 lg:p-8",
      headingContainer: "order-2 lg:order-1",
      heading:
        "font-karla text-2xl lg:text-3xl text-white multi-line-2-white lg:multi-line-3-white group-hover:multi-line-underline leading-8 lg:leading-8 font-extrabold tracking-tight",
      paragraph:
        "order-3 lg:order-2 mt-1 lg:mt-2 font-plex text-sm lg:text-lg text-white leading-4 lg:leading-7 font-normal",
      pillContainer:
        "order-1 lg:order-3 mt-7 flex flex-wrap gap-2 mb-2 lg:mb-0",
    },
  };
  return (
    <div
      className={`${STYLE_MAPPING[mainStyle]["outerContainer"]} ${POSITION_MAPPING[selfAligned]} ${className}`}
    >
      <div className={STYLE_MAPPING[mainStyle]["innerContainer"]}>
        {target ? (
          <Link to={target}>
            <div className={STYLE_MAPPING[mainStyle]["headingContainer"]}>
              <h3 className={STYLE_MAPPING[mainStyle]["heading"]}>{title}</h3>
            </div>
            <p className={STYLE_MAPPING[mainStyle]["paragraph"]}>
              {description}
            </p>
          </Link>
        ) : (
          <>
            <div className={STYLE_MAPPING[mainStyle]["headingContainer"]}>
              <h3 className={STYLE_MAPPING[mainStyle]["heading"]}>{title}</h3>
            </div>
            <p className={STYLE_MAPPING[mainStyle]["paragraph"]}>
              {description}
            </p>
          </>
        )}
        <div className={STYLE_MAPPING[mainStyle]["pillContainer"]}>
          {categories &&
            categories.map((category) => (
              <Pill
                key={category.id}
                title={category.title}
                href={category.href}
                type={transparentOnMobile ? pillType : "dark"}
                size="sm"
                rounded
              />
            ))}
        </div>
      </div>
    </div>
  );
};

