import React, { useEffect, useState, useRef, Fragment } from "react";
import { Pill } from "@components/Pill";
import { CommonUtils } from "@utils/CommonUtils";
import { Close } from "@carbon/icons-react";
import { ChevronRight } from "@carbon/icons-react";


export interface TagCloudProps {
  activeTags?: string[];
  tags?: Array<{ target?: string; title: string }>;
  onTagEnable?: (tag: string) => void;
  onTagDisable?: (tag: string) => void;
}

export const TagCloud: React.FC<TagCloudProps> = ({
  activeTags = [],
  tags,
  onTagEnable = () => {},
  onTagDisable = () => {},
}) => {
  const slidingFactor = 1.2; // Increase/decrease this value to make the tag cloud scroll with wider/narrower width
  const sliderRef = useRef<HTMLDivElement>(null);
  const [scrollX, setScrollX] = useState(0);
  const [isScrolledToRight, setIsScrolledToRight] = useState(false);
  const [isScrolledToLeft, setIsScrolledToLeft] = useState(true);
  const [shouldShowNavigationalButtons, setShouldShowNavigationalButtons] =
    useState(true);

  const handleTagClick = (tag: string) => {
    activeTags.includes(tag) ? onTagDisable(tag) : onTagEnable(tag);
  };

  const handleButtonScroll = (px: number) => {
    if (sliderRef.current) {
      sliderRef.current.scrollTo({
        top: 0,
        left: sliderRef.current.scrollLeft + px,
        behavior: "smooth",
      });
    }
  };

  const handlSwipeScroll = () => {
    if (sliderRef.current) {
      setIsScrolledToLeft(!(sliderRef.current.scrollLeft > 0));
      setIsScrolledToRight(
        sliderRef.current.scrollLeft + sliderRef.current.clientWidth >=
          sliderRef.current.scrollWidth,
      );
    }
  };

  const calculateScrollX = () => {
    if (sliderRef.current) {
      const widthPerSlide =
        (sliderRef.current.scrollWidth - sliderRef.current.clientWidth) *
        slidingFactor;
      setScrollX(widthPerSlide);
    }
  };

  useEffect(() => {
    calculateScrollX();
  }, []);

  useEffect(() => {
    calculateScrollX();
  }, [sliderRef]);

  useEffect(() => {
    if (sliderRef.current) {
      setShouldShowNavigationalButtons(
        sliderRef.current.scrollWidth > sliderRef.current.clientWidth,
      );
    }
  }, [tags]);

  return (
    <div className="relative w-full">
      {shouldShowNavigationalButtons && (
        <Fragment>
          <NavigationalButton
            className={isScrolledToLeft ? "opacity-0 invisible" : "opacity-100"}
            direction="left"
            onClick={() => handleButtonScroll(-scrollX)}
          />
          <NavigationalButton
            className={
              isScrolledToRight ? "opacity-0 invisible" : "opacity-100"
            }
            direction="right"
            onClick={() => handleButtonScroll(scrollX)}
          />
        </Fragment>
      )}
      <div
        ref={sliderRef}
        onScroll={handlSwipeScroll}
        className="w-full flex gap-3 overflow-y-hidden overflow-x-auto whitespace-nowrap no-scrollbar"
      >
        {Boolean(tags?.length) &&
          tags?.map((tag) => {
            const isActiveTag = activeTags.includes(tag.title);
            const tagId = CommonUtils.generateID();
            return isActiveTag ? (
              <Pill
                key={tagId}
                type="dark"
                title={tag.title}
                onClick={() => handleTagClick(tag.title)}
                size="lg"
                icon={
                  <Close className="w-3.5 sm:w-4 h-3.5 sm:h-4 text-white mx-0 sm:mx-1" />
                }
                iconRight
                rounded
              />
            ) : (
              <Pill
                key={tagId}
                type="default"
                title={tag.title}
                onClick={() => handleTagClick(tag.title)}
                size="lg"
                rounded
              />
            );
          })}
      </div>
    </div>
  );
};

interface NavigationalButtonProps {
  direction: "left" | "right";
  onClick: () => void;
  className?: string;
}

const NavigationalButton: React.FC<NavigationalButtonProps> = ({
  direction,
  onClick,
  className = "",
}) => {
  const STYLE_MAPPING: Record<string, string> = {
    left: "hidden sm:block absolute z-[1] h-[101%] -left-1 flex justify-center items-center outline-0 px-3 bg-white after:content-[''] after:absolute after:z-[2] after:left-full after:top-0 after:w-8 after:h-full after:bg-gradient-to-l after:from-transparent after:to-white",
    right:
      "hidden sm:block absolute z-[1] h-[101%] -right-1 flex justify-center items-center outline-0 px-3 bg-white before:content-[''] before:absolute before:z-[2] before:right-full before:top-0 before:w-8 before:h-full before:bg-gradient-to-r before:from-transparent before:to-white",
  };
  return (
    <div className={`${STYLE_MAPPING[direction]} ${className}`}>
      <button onClick={onClick} className="h-full bg-white outline-0">
        <ChevronRight
          size={20}
          className={direction === "left" ? "-scale-100" : "scale-100"}
        />
      </button>
    </div>
  );
};

