import React, { useState } from "react";
import { Link } from "@components/Link";
import { ChevronDown, ChevronRight } from "@carbon/icons-react";
import { ITopBar, ITopBarItem } from "@interfaces/IHeader";
import { LanguageSelector } from "@components/LanguageSelector";
import { Search } from "@components/Search";;

export const TopBar: React.FC<ITopBar> = ({
  items,
  languages,
  onChangeLanguage,
  selectedLanguage,
  onSearch,
  light,
  hideSearchIcon,
  customLogo,
  logoClassName,
}) => {
  const [shouldFadeOthers, setShouldFadeOthers] = useState(false);
  return items.length ? (
    <nav className="w-full relative hidden lg:inline-block bg-white border-b border-black">
      <div
        className={`container flex items-center ${
          customLogo ? "justify-between" : "justify-end"
        }`}
      >
        {Boolean(customLogo) && (
          <div
            className={`w-auto hidden lg:block lg:w-36 lg:h-auto h-8 z-10 overflow-hidden lg:overflow-auto ${
              logoClassName || ""
            }`}
          >
            {customLogo}
          </div>
        )}
        <ul className="flex float-right gap-x-6 w-auto">
          {items.map((item, index) => (
            <TopBarItem
              key={index}
              {...item}
              faded={shouldFadeOthers}
              onHover={(value) => setShouldFadeOthers(value)}
              levelThreeToTheLeft={index >= Math.round((items.length - 1) / 2)}
              firstItem={index === 0}
            />
          ))}
          {!hideSearchIcon && (
            <li className="flex items-center xl:ml-20">
              <Search
                onSearch={(keyword) => onSearch && onSearch(keyword)}
                faded={shouldFadeOthers}
                onHover={(value) => setShouldFadeOthers(value)}
              />
            </li>
          )}
          {languages && languages?.length > 1 ? (
            <li className="flex items-center ml-5">
              <LanguageSelector
                languages={languages}
                selectedLanguage={selectedLanguage}
                onChangeLanguage={onChangeLanguage}
                faded={shouldFadeOthers}
                onHover={(value) => setShouldFadeOthers(value)}
                light={light}
              />
            </li>
          ) : null}
        </ul>
      </div>
    </nav>
  ) : null;
};

const TopBarItem: React.FC<ITopBarItem> = ({
  title,
  target,
  children,
  levelThreeToTheLeft,
  faded,
  onHover,
  firstItem,
}) => {
  const classes =
    "group py-2 relative text-sm lg:text-base leading-6 font-medium flex-1 text-center xl:text-right whitespace-nowrap xl:ml-20 transition-all ease-in duration-100 hover:text-grey-900";
  return (
    <li
      onMouseEnter={() => onHover && onHover(true)}
      onMouseLeave={() => onHover && onHover(false)}
      className={`${classes} ${faded ? "text-grey-400" : "text-grey-900"} ${target ? "cursor-pointer" : ""}`}
    >
      {target ? (
        <Link to={target}>
          <span className="flex items-center justify-center xl:justify-end">
            {title}
            {children && children.length > 0 && (
              <ChevronDown size={16} className="ml-2 group-hover:opacity-0" />
            )}
          </span>
        </Link>
      ) : (
        <div>
          <span className="flex items-center justify-center xl:justify-end">
            {title}
            {children && children.length > 0 && (
              <ChevronDown size={16} className="ml-2 group-hover:opacity-0" />
            )}
          </span>
        </div>
      )}
      {children && children.length > 0 && (
        <div
          className={`z-20 group-hover:block hidden absolute top-10 xl:-left-7 w-[273px] bg-grey-900 ${
            firstItem ? "-left-4" : "left-5"
          }`}
        >
          <div className="flex flex-col items-start font-plex text-base leading-6 font-medium text-grey-600">
            {children.map((levelTwoChild, index) => {
              const linkClasses = `peer relative block w-full px-7 whitespace-normal ${
                index === 0
                  ? "pt-7 pb-3"
                  : index === children.length - 1
                    ? "pt-3 pb-7"
                    : "py-3"
              } ${children.length === 1 ? "pb-7" : ""}`;
              return (
                <div
                  key={index}
                  className="w-full relative text-left hover:text-white"
                >
                  {levelTwoChild.target ? (
                    <Link className={linkClasses} to={levelTwoChild.target}>
                      {levelTwoChild.title}
                      {levelTwoChild.children &&
                        levelTwoChild.children.length > 0 && (
                          <ChevronRight
                            size={20}
                            className={`absolute right-3 -translate-y-2/4 ${
                              index === 0
                                ? "top-[65%]"
                                : index === children.length - 1
                                  ? "top-[40%]"
                                  : "top-1/2"
                            }`}
                          />
                        )}
                    </Link>
                  ) : (
                    <span className={linkClasses}>
                      {levelTwoChild.title}
                      {levelTwoChild.children &&
                        levelTwoChild.children.length > 0 && (
                          <ChevronRight
                            size={20}
                            className={`absolute right-3 -translate-y-2/4 ${
                              index === 0
                                ? "top-[65%]"
                                : index === children.length - 1
                                  ? "top-[40%]"
                                  : "top-1/2"
                            }`}
                          />
                        )}
                    </span>
                  )}

                  {levelTwoChild.children &&
                    levelTwoChild.children.length > 0 && (
                      <div
                        className={`peer-hover:block hover:block hidden absolute z-10 w-[274px] ${
                          index > 0 ? "-top-4" : "top-0"
                        } ${levelThreeToTheLeft ? "right-full" : "left-full"}`}
                      >
                        <div className="flex flex-col items-start bg-grey-900 font-plex text-base leading-6 font-medium text-grey-600">
                          {levelTwoChild.children?.map(
                            (levelThreeChild, index) => (
                              <div
                                key={index}
                                className="w-full text-left hover:text-white"
                              >
                                <Link
                                  className={`block w-full relative px-7 whitespace-normal ${
                                    index === 0
                                      ? "pt-7 pb-3"
                                      : index ===
                                          (levelTwoChild.children &&
                                            levelTwoChild.children.length - 1)
                                        ? "pt-3 pb-7"
                                        : "py-3"
                                  } ${
                                    levelTwoChild.children?.length === 1
                                      ? "pb-7"
                                      : ""
                                  }`}
                                  to={levelThreeChild.target}
                                >
                                  {levelThreeChild.title}
                                </Link>
                              </div>
                            ),
                          )}
                        </div>
                      </div>
                    )}
                </div>
              );
            })}
          </div>
        </div>
      )}
    </li>
  );
};
